        LIST P=12F629,R=HEX
;       RC-Switch  4 	
;	VERSION 1.0
;	:
;   1,0-1,2  -   1
;   1,2-1,4  -  2
;   1,4-1,6  -   
;   1,6-1,8  -  3
;   1,8-2,0  -  4
;	  1    2  -   
;	 :
;	  
;	  
;	CHIP CONFIG : INTERC, WDT OFF 
;	
;===================================================       
;       DEFINE RAM
;-----  SYSTEM  ---------
TMR0          EQU      1	    ;TIMER 0
PCL   		  EQU      2       ;PROGRAM COUNTER
STATUS        EQU      3
FSR           EQU      4       ;BUFFER OF INDIRECT ADDR
GPIO          EQU      5
OPTION_reg    EQU      0x81
TRISIO	      EQU      0x85
OSCCAL        EQU      0x90	
RP0	          EQU      5
CMCON	      EQU      0x19	   ;CMCON REGISTER
ANSEL         EQU      0x9F	   ;ANALOG SEL
CM0           EQU      0
CM1           EQU      1
CM2           EQU      2
W             EQU      0           
F             EQU      1    
C             EQU      0           
Z             EQU      2  

;----  USER registers DEFINE  ------
TMR3          EQU      H'22'    ;PWM WIDTH, INPUT
MODE	      EQU      H'2B'	; 
NEWMODE	      EQU      H'2C'	; 
PREVMODE	  EQU      H'2D'	; 
MODECNT	      EQU      H'2E'	; 
MODE_FLAG     EQU      0		;FLAG IN FLAG_REG

;----  VARIABLES DEFINE ----
MIN_PWM	      EQU      D'100'
CMD1_PWM	  EQU      D'120'
CMD2_PWM	  EQU      D'140'
CMD0_PWM	  EQU      D'160'
CMD3_PWM	  EQU      D'180'
MAX_PWM	      EQU      D'200'
DELAY	      EQU      D'20'

;----  HARDWARE PINS DEFINE ----
INPIN         EQU      3        ;GPIO 3 = INPUT PIN 
SETPIN	      EQU      0		;GPIO 0, OPEN = MODE 0, GROUNDING = MODE 1
OUT1	      EQU      1		;OUTPUT 1 = GPIO 1
OUT2	      EQU      2	
OUT3	      EQU      4	
OUT4	      EQU      5	

;----  SYSTEM CONFIG  ----
   	errorlevel    -302         ; suppress message 302 from list file
	__CONFIG	H'3F84'                ;_CP_OFF & _CPD_OFF & _BODEN_OFF & _MCLRE_OFF & _WDT_OFF & _PWRTE_ON & _INTRC_OSC_NOCLKOUT 
;==============================================
;==========   MAIN PROGRAM   =========
;------------------------------------
;INPUT PARAMETERS,
;TEMP REGISTERS 
;OUTPUT PARAMETERS
;------------------------------------
;--- RESTORE OSC CALIBRATION ---------
	    ORG           0
	    BSF           STATUS,RP0	;;BANK1
	    CALL          3FFh
	    MOVWF         OSCCAL	    ;RESTORE OSC CALI	
	    MOVLW         B'00001001'   ;GPIO 0&3=INPUT, OTHERS =OUTPUT		
	    MOVWF         TRISIO 
	    MOVLW         B'01000000'   ;GPPU=0 TO ENBALE PULL-UP, T0CS=0, TO ENABLE GP2
	    MOVWF         OPTION_reg 	
	    CLRF          ANSEL	        ;Turn off ADC on the 12F675 - All pins digital
	    BCF           STATUS,RP0	;BANK0
	    BSF           CMCON,CM2	    ;turn the input pins back to digital
	    BSF           CMCON,CM1
	    BSF           CMCON,CM0
	    CLRF 	      GPIO	        ;SET ALL OUTPUT LOW 
	    CLRF	      MODE
	    CLRF	      NEWMODE
	    CLRF	      PREVMODE
	    CLRF	      MODECNT
	    CALL	      DISABLE_OLL        
	    BTFSS	      GPIO,SETPIN	;SET MODE

;==================================================================================================================
;         1/7     0.8 ,   
;==================================================================================================================
MAIN	
	   CALL       GET_PWM

	   MOVLW      MIN_PWM	     ;   
	   CALL	      CHECK_PWM
	   BTFSC	  STATUS,C
	   GOTO	      CMD1
	   BTFSC	  STATUS,Z
	   GOTO	      MAIN
	   CALL	      DISABLE_OLL	; 
	   GOTO       MAIN
;==================================================================================================================
;       2/7  1 ,    GPIO,OUT1
;==================================================================================================================
CMD1                               ;   1     OUT1
	   MOVLW      CMD1_PWM
	   CALL	      CHECK_PWM
	   BTFSC	  STATUS,C
	   GOTO	      CMD2
	   BTFSC	  STATUS,Z
	   GOTO	      MAIN
	   CALL	      DISABLE_OLL	; 
	   GOTO       MAIN
;==================================================================================================================
;       3/7  1.3 ,    GPIO,OUT1  OUT2
;==================================================================================================================	
CMD2                        ;   1.3     OUT1+2
	  MOVLW       CMD2_PWM
	  CALL	      CHECK_PWM
	  BTFSC	      STATUS,C
	  GOTO	      CMD3             ;  	GOTO	CMD0
	  BTFSC	      STATUS,Z
	  GOTO	      MAIN
	   BSF	       GPIO,OUT1
	   BSF	       GPIO,OUT2
	   BCF 	       GPIO,OUT3
	   BCF	       GPIO,OUT4
	   GOTO        MAIN
;==================================================================================================================
;       4/7  1.5  ( c 3- , )    GPIO,OUT1,OUT2
;==================================================================================================================	
CMD0                       ;  1.5
	   MOVLW       CMD0_PWM
	   CALL	       CHECK_PWM
	   BTFSC	   STATUS,C
	   GOTO	       CMD3
	   BTFSC	   STATUS,Z
	   GOTO	       MAIN
	   BSF	       GPIO,OUT1
	   BSF	       GPIO,OUT2
	   BCF 	       GPIO,OUT3
	   BCF	       GPIO,OUT4
	   GOTO        MAIN
;==================================================================================================================
;       5/7  1.7      GPIO,OUT1,OUT3
;==================================================================================================================	
CMD3                       ;  1.7    OUT1+2+3
	  MOVLW       CMD3_PWM
      CALL	      CHECK_PWM
	  BTFSC	      STATUS,C
	  GOTO	      CMD4
	  BTFSC	      STATUS,Z
	  GOTO	      MAIN
	   BSF	       GPIO,OUT1
	   BSF	       GPIO,OUT2
	   BCF 	       GPIO,OUT3
	   BCF	       GPIO,OUT4
	   GOTO        MAIN
;==================================================================================================================
;       6/7  2      GPIO,OUT1,OUT3,OUT4
;==================================================================================================================	
CMD4                      ;  2    OUT1+2+3+4
	  MOVLW       MAX_PWM
	  CALL	      CHECK_PWM
	  BTFSC	      STATUS,C
	  GOTO	      MAIN         ;  - OVF
	  BTFSC	      STATUS,Z
	  GOTO	      MAIN
	  BSF	      GPIO,OUT1
	  BSF	      GPIO,OUT2
	  BSF	      GPIO,OUT3
	  BSF	      GPIO,OUT4
	  GOTO        MAIN
;==================================================================================================================
;       7/7  2.2      GPIO,OUT1,OUT3,OUT4 ( )
;==================================================================================================================	
OVF                      ;  2.2
	  MOVLW       D'255'
	  DECF	      TMR3,F
	  CALL	      CHECK_PWM
	  BTFSC	      STATUS,Z
	  GOTO	      MAIN
	  BSF	      GPIO,OUT1
	  BSF	      GPIO,OUT2
	  BSF	      GPIO,OUT3
	  BSF	      GPIO,OUT4
	  GOTO        MAIN
;==================================================================================================================
;==================================================================================================================	
GET_PWM	
	  BTFSS      GPIO,INPIN   ;IF INPIN = HIGH, INCREASE TMR3
      GOTO       GET_PWM
	  CLRF       TMR3	      ;READ INPIN PULSE WEITH

GET_PWM2
	  INCF       TMR3,F        ;START LOOP
   	  NOP
      NOP                       ;10 uS LOOP
      NOP
      NOP
      NOP
      NOP
	  BTFSC      GPIO,INPIN	    ;UNTIL INPIN GO LOW
      GOTO       GET_PWM2       ;END LOOP
	  RETURN
;-----------------------------------------------
;==================================================================================================================
;               0.8
;==================================================================================================================	
DISABLE_OLL
	  BCF	    GPIO,OUT1
	  BCF	    GPIO,OUT2
	  BCF	    GPIO,OUT3
	  BCF	    GPIO,OUT4
	  RETURN
;==================================================================================================================
;==================================================================================================================	
;-------  TMR3  W------------
CHECK_PWM
	MOVWF	 NEWMODE
	SUBWF    TMR3,W
	BTFSC	 STATUS,C
	RETURN	                          ;    - 
	MOVF	 NEWMODE,W
	SUBWF	 MODE,W
	BTFSC	 STATUS,Z
	GOTO	 MATCH
                                    	;     
	MOVF	 NEWMODE,W
	SUBWF	 PREVMODE,W
	BTFSC	 STATUS,Z
	GOTO 	 MATCHPREV_NOMATCHMODE
                                     	;     
	MOVF	 NEWMODE,W
	MOVWF	 PREVMODE	                ;  
	MOVLW	 DELAY
	MOVWF	 MODECNT	                ;  
NOCHANGE
	BCF	     STATUS,C	                ; 
	BSF	     STATUS,Z	                ;  
	RETURN

MATCHPREV_NOMATCHMODE                    ;    
	DECFSZ	 MODECNT	                 ;  
	GOTO	 NOCHANGE	                 ;  
                                         ;   
	MOVF	 NEWMODE,W
	MOVWF	 PREVMODE	                 ;  
	MOVWF	 MODE	                     ;  
	MOVLW	 DELAY
	MOVWF	 MODECNT	                 ;  
	BCF	     STATUS,C	                 ; 
	BCF	     STATUS,Z	                 ;    
	RETURN
MATCH
	MOVF	 NEWMODE,W
	MOVWF	 PREVMODE	                 ;  
	MOVWF	 MODE	                     ;  
	MOVLW	 DELAY
	MOVWF	 MODECNT	                 ;  
	BCF	     STATUS,C	                 ; 
	BCF	     STATUS,Z	                 ;    
	RETURN

;---------------------------------------------------
        END


